
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <sys/time.h>

#include "lights.h"

#define OUT_POS 1000.0

static int c_id=0;

Lights::Lights( ssgRoot *scene, double x, double y, double z, double a )
{
	id = ++c_id;
	_red_light       = new ACObject(scene, "red_light.ac");	
	_orange_light    = new ACObject(scene, "orange_light.ac");
	_green_light     = new ACObject(scene, "green_light.ac");
	_redorange_light = new ACObject(scene, "redorange_light.ac");
	_no_light        = new ACObject(scene, "no_light.ac");

	sw_mode=act_mode=GO;
	lx=x; ly=y; lz=z; la=a;
	changing_light=0;
	flipflop=0;
}

void Lights::render()
{
	if( act_mode != sw_mode ) {
		if(!changing_light)timer=time(NULL);
		changing_light=1;
	} else {
		changing_light=0;
	}

//	printf("act_mode=%d sw_mode=%d changing_light=%d\n",act_mode,sw_mode,changing_light);
	if( changing_light ) {
		switch ( act_mode ) {
			case GO:
				if( sw_mode == NIGHT_TRAFFIC ) act_mode=NIGHT_TRAFFIC;
				if( sw_mode == STOP )          _green2red(); 
				break;
			case STOP:
				if( sw_mode == NIGHT_TRAFFIC ) act_mode=NIGHT_TRAFFIC;
				if( sw_mode == GO )            _red2green(); 
				break;
			case NIGHT_TRAFFIC:
				if( sw_mode == GO )    act_mode=GO; 
				if( sw_mode == STOP )  act_mode=STOP; 
				break;
		}
		return;
	}
	//printf("render(): act_mode=%d\n", act_mode);
	switch(act_mode){
		case NIGHT_TRAFFIC:
			_night_traffic();
			break;
		case GO:
			green_light();
			break;	
		case STOP:
			red_light();	
	}
}

		

/* This is FSA for Semaphore */
void Lights::set_state( lights_mode_t mode )
{
	sw_mode = mode;
}


/* night traffic mode */
void Lights::_night_traffic()
{
	if( time(NULL) - timer >= DEFAULT_NIGHT_DELAY ){
		flipflop^=1;	
		timer=time(NULL);
	}

	if(flipflop) orange_light();
	else         no_light();

	act_mode=NIGHT_TRAFFIC;
}

/* green, orange, red */
void Lights::_green2red()
{
	orange_light();	
	if( time(NULL)-timer >= DEFAULT_DELAY ) {
		red_light();
		act_mode=STOP;
	}
}

/* red light, red/orange, green */
void Lights::_red2green()
{
	redorange_light();	
	if( time(NULL)-timer > DEFAULT_NIGHT_DELAY ) {
		green_light();	
		act_mode = GO;
	}
}

void Lights::red_light()
{
	ssgGetLight(id)->setPosition( lx, ly-15, lz+45 );
	ssgGetLight(id)->setSpotDirection( lx, ly, lz+36 );
	ssgGetLight(id)->setSpotAttenuation( 0, 0.01, 0 );
    ssgGetLight(id)->setSpotDiffusion( 0.9, 30 );
    ssgGetLight(id)->setSpotlight( 1 );
	ssgGetLight(id)->on();

	_red_light      ->set_position( lx, ly, lz, la );
	_orange_light   ->set_position(OUT_POS,OUT_POS,OUT_POS);
	_no_light       ->set_position(OUT_POS,OUT_POS,OUT_POS);
	_redorange_light->set_position(OUT_POS,OUT_POS,OUT_POS);
	_green_light    ->set_position(OUT_POS,OUT_POS,OUT_POS);
}

void Lights::green_light()
{
	ssgGetLight(id)->setPosition( lx, ly-15, lz+32 );
	ssgGetLight(id)->setSpotDirection( lx, ly, lz+22 );
	ssgGetLight(id)->setSpotAttenuation( 0, 0.01, 0 );
    ssgGetLight(id)->setSpotDiffusion( 0.9, 30 );
    ssgGetLight(id)->setSpotlight( 1 );
	ssgGetLight(id)->on();

	_green_light    ->set_position( lx, ly, lz, la );
	_orange_light   ->set_position(OUT_POS,OUT_POS,OUT_POS);
	_no_light       ->set_position(OUT_POS,OUT_POS,OUT_POS);
	_redorange_light->set_position(OUT_POS,OUT_POS,OUT_POS);
	_red_light      ->set_position(OUT_POS,OUT_POS,OUT_POS);
}

void Lights::orange_light() 
{
	ssgGetLight(id)->setPosition( lx, ly-15, lz+32 );
	ssgGetLight(id)->setSpotDirection( lx, ly, lz+32 );
	ssgGetLight(id)->setSpotAttenuation( 0, 0.01, 0 );
    ssgGetLight(id)->setSpotDiffusion( 0.9, 30 );
    ssgGetLight(id)->setSpotlight( 1 );
	ssgGetLight(id)->on();
	_orange_light   ->set_position( lx, ly, lz, la );
	_green_light    ->set_position(OUT_POS,OUT_POS,OUT_POS);
	_no_light       ->set_position(OUT_POS,OUT_POS,OUT_POS);
	_redorange_light->set_position(OUT_POS,OUT_POS,OUT_POS);
	_red_light      ->set_position(OUT_POS,OUT_POS,OUT_POS);
}

void Lights::redorange_light()
{
	ssgGetLight(id)->setPosition( lx, ly-15, lz+40 );
	ssgGetLight(id)->setSpotDirection( lx, ly, lz+45 );
	ssgGetLight(id)->setSpotAttenuation( 0, 0.01, 0 );
    ssgGetLight(id)->setSpotDiffusion( 0.9, 30 );
    ssgGetLight(id)->setSpotlight( 1 );
	ssgGetLight(id)->on();

	_redorange_light->set_position( lx, ly, lz, la );
	_green_light    ->set_position(OUT_POS,OUT_POS,OUT_POS);
	_no_light       ->set_position(OUT_POS,OUT_POS,OUT_POS);
	_orange_light   ->set_position(OUT_POS,OUT_POS,OUT_POS);
	_red_light      ->set_position(OUT_POS,OUT_POS,OUT_POS);
}

void Lights::no_light()
{
	_no_light       ->set_position( lx, ly, lz, la );
	_green_light    ->set_position(OUT_POS,OUT_POS,OUT_POS);
	_orange_light   ->set_position(OUT_POS,OUT_POS,OUT_POS);
	_redorange_light->set_position(OUT_POS,OUT_POS,OUT_POS);
	_red_light      ->set_position(OUT_POS,OUT_POS,OUT_POS);
}

void Lights::set_position(double x, double y, double z, double a)
{
	lx=x; ly=y; lz=z; la=a;
}
void Lights::set_position(double x, double y, double z)
{
	lx=x; ly=y; lz=z; 
}
