#include "sky.h"
#include "timer.h"

#define SKY_DISTANCE 200000
#define RADIUS ((SKY_DISTANCE)/30.0)

static	sgVec4 _base_ambient      = { 0.2f, 0.2f, 0.2f, 1.0f } ;
static	sgVec4 _base_diffuse      = { 1.0f, 1.0f, 1.0f, 1.0f } ;
static	sgVec4 _base_specular     = { 1.0f, 1.0f, 1.0f, 1.0f } ;
static	sgVec4 _sky_color    = { 0.39f, 0.50f, 0.74f, 1.0f } ;
static	sgVec4 _fog_color    = { 0.84f, 0.87f, 0.87f, 0.8f } ;
//static	sgVec4 _fog_color    = { 0.84f, 0.87f, 1.00f, 1.0f } ;
static	sgVec4 _black             = { 0.0f, 0.2f, 0.4f, 1.0f } ;


MySky::MySky()
{

	timer=new Timer(30);
	
	bodies[0]=bodies[1]=NULL;

	 
	sgCopyVec4( base_ambient, _base_ambient);
	sgCopyVec4( base_diffuse, _base_diffuse);
	sgCopyVec4( base_specular, _base_specular);
	sgCopyVec4( base_sky_color, _sky_color);
	sgCopyVec4( base_fog_color, _fog_color);
	sgCopyVec4( black, _black);

	nstars=1000  ;
	star_data = new sgdVec3[nstars] ;
	if(star_data==NULL){
		fprintf(stderr,"Cannot alloc vector for stars!\n");
		exit(-1);
	}
	for ( int i = 0; i < nstars; i++ )  {
		star_data[i][0] = ssgaRandom() * SGD_PI ;
		star_data[i][1] = ssgaRandom() * SGD_PI ;
		star_data[i][2] = ssgaRandom() ;
	}


	sky       =  new ssgaSky ;
	sky       -> build ( SKY_DISTANCE, SKY_DISTANCE, 0, NULL /*planet_data*/, nstars, star_data );
//	clouds=new ssgaCloudLayer();
//	clouds->build("data/clouds.rgb", 0.0, 20000.0, 2000.0, 1.0 );
//	clouds->setSpeed(50);
//	clouds->setDirection(20);
//	clouds->enable();

	bodies[0] =  sky -> addBody (
                  NULL,                 // body texture
				  "data/halo.rgba",     // halo texture
				  RADIUS,                 // size
				  SKY_DISTANCE,                // distance
				  true );               // is sun - dome painted based on this
	bodies[0] -> setDeclination  ( -50*SGD_DEGREES_TO_RADIANS );
  /* moon */
	bodies[1] =  sky -> addBody (
                  "data/moon.rgba",     // body texture
				  NULL,                 // halo texture
				  RADIUS,                 // size
				  SKY_DISTANCE );              // distance
	bodies[1] -> setDeclination  ( 165*SGD_DEGREES_TO_RADIANS );
}

MySky::~MySky() { }

void MySky::preDraw() 
{ 
	sky->preDraw(); 
//	clouds->draw(); 
//	clouds->repaint(sky_color); 
}

void MySky::postDraw(double z) { sky->postDraw(z); }

void MySky::update_sky( sgCoord campos )
{
  double sol_angle = bodies[0]->getAngle();
  double sky_brightness = (1.0 + cos(sol_angle))/2.0; // 0.0 - 1.0
  double scene_brightness = pow(sky_brightness,0.5);

  /* set sky color */
  sky_color[0] = base_sky_color[0] * (float)sky_brightness;
  sky_color[1] = base_sky_color[1] * (float)sky_brightness;
  sky_color[2] = base_sky_color[2] * (float)sky_brightness;
  sky_color[3] = base_sky_color[3];


  cloud_color[0] = fog_color[0] = base_fog_color[0] * (float)sky_brightness;
  cloud_color[1] = fog_color[1] = base_fog_color[1] * (float)sky_brightness;
  cloud_color[2] = fog_color[2] = base_fog_color[2] * (float)sky_brightness;
  cloud_color[3] = fog_color[3] = base_fog_color[3];



  sky -> repositionFlat ( campos.xyz, 0, timer->get_game_time_delta() );
  sky -> modifyVisibility ( campos.xyz[SG_Z], 0 );
  sky -> repaint ( sky_color, fog_color, cloud_color, sol_angle, 0, NULL, nstars, star_data );
  sgCoord solpos;
  bodies[0] -> getPosition ( & solpos );
  ssgGetLight ( 0 ) -> setPosition ( solpos.xyz ) ;

  scene_ambient[0] = base_ambient[0] * (float)scene_brightness;
  scene_ambient[1] = base_ambient[1] * (float)scene_brightness;
  scene_ambient[2] = base_ambient[2] * (float)scene_brightness;
  scene_ambient[3] = 1.0;

  scene_diffuse[0] = base_diffuse[0] * (float)scene_brightness;
  scene_diffuse[1] = base_diffuse[1] * (float)scene_brightness;
  scene_diffuse[2] = base_diffuse[2] * (float)scene_brightness;
  scene_diffuse[3] = 1.0;

  scene_specular[0] = base_specular[0] * (float)scene_brightness;
  scene_specular[1] = base_specular[1] * (float)scene_brightness;
  scene_specular[2] = base_specular[2] * (float)scene_brightness;
  scene_specular[3] = 1.0;
  // GL_LIGHT_MODEL_AMBIENT has a default non-zero value so if
  // we only update GL_AMBIENT for our lights we will never get
  // a completely dark scene.  So, we set GL_LIGHT_MODEL_AMBIENT
  // explicitely to black.
  glLightModelfv( GL_LIGHT_MODEL_AMBIENT, black );
  ssgGetLight( 0 ) -> setColour( GL_AMBIENT, scene_ambient );
  ssgGetLight( 0 ) -> setColour( GL_DIFFUSE, scene_diffuse );
  ssgGetLight( 0 ) -> setColour( GL_SPECULAR, scene_specular );
}

void MySky::set_sun_dec( double e )
{
	bodies[0] -> setDeclination  ( e*SGD_DEGREES_TO_RADIANS );
}

void MySky::set_moon_dec( double e )
{
	bodies[1] -> setDeclination  ( e*SGD_DEGREES_TO_RADIANS );
}
